"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const enumerable_1 = require("../enumerable");
const equality_comparer_1 = require("../equality-comparer");
const enumerable_container_1 = require("./enumerable-container");
const utils = require("../utils");
function groupJoin(outer, inner, outerKeySelector, innerKeySelector, resultSelector, comparer) {
    utils.throws.ThrowIfNull('outer', outer);
    utils.throws.ThrowIfNull('inner', inner);
    utils.throws.ThrowIfNull('outerKeySelector', outerKeySelector);
    utils.throws.ThrowIfNull('innerKeySelector', innerKeySelector);
    let _resultSelector = resultSelector ||
        ((o, is) => {
            return { Outer: o, Inners: is };
        });
    comparer = comparer || equality_comparer_1.EqualityComparer.Default();
    let outerKeys = enumerable_1.Enumerable.ToList(enumerable_1.Enumerable.Select(outer, item => outerKeySelector(item)));
    let groupedInner = enumerable_1.Enumerable.ToList(enumerable_1.Enumerable.GroupBy(enumerable_1.Enumerable.Where(inner, item => enumerable_1.Enumerable.Contains(outerKeys, innerKeySelector(item), comparer)), item => innerKeySelector(item), g => g, comparer));
    return new enumerable_container_1.EnumerableContainer([
        ...join(outer, outerKeySelector, groupedInner, _resultSelector, comparer)
    ]);
}
exports.groupJoin = groupJoin;
function* join(outer, outerKeySelector, groupedInner, resultSelector, comparer) {
    for (let item of outer) {
        let key = outerKeySelector(item);
        let inners = enumerable_1.Enumerable.FirstOrDefault(groupedInner, null, g => comparer.Equals(g.Key, key));
        if (!inners) {
            inners = enumerable_1.Enumerable.Empty();
        }
        else {
            inners = enumerable_1.Enumerable.ToList(inners);
        }
        yield resultSelector(item, inners);
    }
}
