"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const enumerable_1 = require("../enumerable");
const equality_comparer_1 = require("../equality-comparer");
const enumerable_container_1 = require("./enumerable-container");
const utils = require("../utils");
function join(outer, inner, outerKeySelector, innerKeySelector, resultSelector, comparer) {
    utils.throws.ThrowIfNull('outer', outer);
    utils.throws.ThrowIfNull('inner', inner);
    utils.throws.ThrowIfNull('outerKeySelector', outerKeySelector);
    utils.throws.ThrowIfNull('innerKeySelector', innerKeySelector);
    let _resultSelector = resultSelector ||
        ((o, i) => {
            return { Outer: o, Inner: i };
        });
    comparer = comparer || equality_comparer_1.EqualityComparer.Default();
    let outerKeys = enumerable_1.Enumerable.ToList(enumerable_1.Enumerable.Select(outer, item => outerKeySelector(item)));
    let groupedInner = enumerable_1.Enumerable.ToList(enumerable_1.Enumerable.GroupBy(enumerable_1.Enumerable.Where(inner, item => enumerable_1.Enumerable.Contains(outerKeys, innerKeySelector(item), comparer)), item => innerKeySelector(item), g => g, comparer));
    return new enumerable_container_1.EnumerableContainer([
        ...process(outer, outerKeySelector, groupedInner, _resultSelector, comparer)
    ]);
}
exports.join = join;
function* process(outer, outerKeySelector, groupedInner, resultSelector, comparer) {
    for (let item of outer) {
        let key = outerKeySelector(item);
        let inners = enumerable_1.Enumerable.FirstOrDefault(groupedInner, null, g => comparer.Equals(g.Key, key));
        if (inners && enumerable_1.Enumerable.Any(inners)) {
            for (let inner of inners) {
                yield resultSelector(item, inner);
            }
        }
    }
}
