"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const enumerable_1 = require("./enumerable");
const exceptions_1 = require("./exceptions");
class List {
    constructor(collection) {
        this.items = [];
        if (collection) {
            if (collection instanceof Array) {
                this.items = collection.map(x => x);
            }
            else {
                this.AddRange(collection);
            }
        }
    }
    get Length() {
        return this.items.length;
    }
    get IsReadOnly() {
        return false;
    }
    Set(index, item) {
        if (index < 0 || index >= this.Length) {
            throw new exceptions_1.ArgumentOutOfRangeException('index', index);
        }
        this.items[index] = item;
    }
    Get(index) {
        if (index < 0 || index >= this.Length) {
            throw new exceptions_1.ArgumentOutOfRangeException('index', index);
        }
        return this.items[index];
    }
    Add(item) {
        this.items.push(item);
    }
    AddRange(collection) {
        if (collection) {
            for (let item of collection) {
                this.Add(item);
            }
        }
    }
    Clear() {
        this.items = [];
    }
    CopyTo(array, arrayIndex) {
        let i = 0;
        for (let item of this) {
            array[i++ + arrayIndex] = item;
        }
    }
    IndexOf(item) {
        return this.items.findIndex(x => x === item);
    }
    Insert(index, item) {
        if (index < 0 || index >= this.Length) {
            throw new exceptions_1.ArgumentOutOfRangeException('index', index);
        }
        this.items.splice(index, 0, item);
    }
    Remove(item) {
        let index = this.IndexOf(item);
        if (index >= 0) {
            this.RemoveAt(index);
            return true;
        }
        return false;
    }
    RemoveAt(index) {
        if (index < 0 || index >= this.Length) {
            throw new exceptions_1.ArgumentOutOfRangeException('index', index);
        }
        this.items.splice(index, 1);
    }
    *[Symbol.iterator]() {
        for (let item of this.items) {
            yield item;
        }
    }
    All(predicate) {
        return enumerable_1.Enumerable.All(this, predicate);
    }
    Any(predicate) {
        return enumerable_1.Enumerable.Any(this, predicate);
    }
    AsEnumerable() {
        return enumerable_1.Enumerable.AsEnumerable(this);
    }
    Average(selector) {
        return enumerable_1.Enumerable.Average(this, selector);
    }
    Concat(second) {
        return enumerable_1.Enumerable.Concat(this, second);
    }
    Contains(value, comparer) {
        return enumerable_1.Enumerable.Contains(this, value, comparer);
    }
    Count(predicate) {
        return enumerable_1.Enumerable.Count(this, predicate);
    }
    DefaultIfEmpty(defaultValue) {
        return enumerable_1.Enumerable.DefaultIfEmpty(this, defaultValue);
    }
    Distinct(comparer) {
        return enumerable_1.Enumerable.Distinct(this, comparer);
    }
    ElementAt(index) {
        return enumerable_1.Enumerable.ElementAt(this, index);
    }
    ElementAtOrDefault(defaultValue, index) {
        return enumerable_1.Enumerable.ElementAtOrDefault(this, defaultValue, index);
    }
    Except(second, comparer) {
        return enumerable_1.Enumerable.Except(this, second, comparer);
    }
    First(predicate) {
        return enumerable_1.Enumerable.First(this, predicate);
    }
    FirstOrDefault(defaultValue, predicate) {
        return enumerable_1.Enumerable.FirstOrDefault(this, defaultValue, predicate);
    }
    GroupBy(keySelector, elementSelector, comparer) {
        return enumerable_1.Enumerable.GroupBy(this, keySelector, elementSelector, comparer);
    }
    GroupJoin(inner, outerKeySelector, innerKeySelector, resultSelector, comparer) {
        return enumerable_1.Enumerable.GroupJoin(this, inner, outerKeySelector, innerKeySelector, resultSelector, comparer);
    }
    Intersect(second, comparer) {
        return enumerable_1.Enumerable.Intersect(this, second, comparer);
    }
    Join(inner, outerKeySelector, innerKeySelector, resultSelector, comparer) {
        return enumerable_1.Enumerable.Join(this, inner, outerKeySelector, innerKeySelector, resultSelector, comparer);
    }
    Last(predicate) {
        return enumerable_1.Enumerable.Last(this, predicate);
    }
    LastOrDefault(defaultValue, predicate) {
        return enumerable_1.Enumerable.LastOrDefault(this, defaultValue, predicate);
    }
    Max(selector) {
        return enumerable_1.Enumerable.Max(this, selector);
    }
    Min(selector) {
        return enumerable_1.Enumerable.Min(this, selector);
    }
    Reverse() {
        return enumerable_1.Enumerable.Reverse(this);
    }
    Select(selector) {
        return enumerable_1.Enumerable.Select(this, selector);
    }
    SelectMany(collectionSelector, resultSelector) {
        return enumerable_1.Enumerable.SelectMany(this, collectionSelector, resultSelector);
    }
    SequenceEqual(second, comparer) {
        return enumerable_1.Enumerable.SequenceEqual(this, second, comparer);
    }
    Single(predicate) {
        return enumerable_1.Enumerable.Single(this, predicate);
    }
    SingleOrDefault(defaultValue, predicate) {
        return enumerable_1.Enumerable.SingleOrDefault(this, defaultValue, predicate);
    }
    Skip(count) {
        return enumerable_1.Enumerable.Skip(this, count);
    }
    SkipWhile(predicate) {
        return enumerable_1.Enumerable.SkipWhile(this, predicate);
    }
    Sum(selector) {
        return enumerable_1.Enumerable.Sum(this, selector);
    }
    Take(count) {
        return enumerable_1.Enumerable.Take(this, count);
    }
    TakeWhile(predicate) {
        return enumerable_1.Enumerable.TakeWhile(this, predicate);
    }
    ToArray() {
        return enumerable_1.Enumerable.ToArray(this);
    }
    ToDictionary(keySelector, elementSelector, comparer) {
        return enumerable_1.Enumerable.ToDictionary(this, keySelector, elementSelector, comparer);
    }
    ToList() {
        return enumerable_1.Enumerable.ToList(this);
    }
    Union(second, comparer) {
        return enumerable_1.Enumerable.Union(this, second, comparer);
    }
    Where(predicate) {
        return enumerable_1.Enumerable.Where(this, predicate);
    }
}
exports.List = List;
